/*
** endian.h - Endian conversion routines
**
** (C) Copyright 2002 BEAM Ltd.
*/

#ifndef _BEAM_ADMXRC2_ENDIAN_H
#define _BEAM_ADMXRC2_ENDIAN_H

#include <stdint.h>
#include <endian.h>

/* Only use this if the system does not supply appropriate endian macros */
#ifndef cpu_to_le32

#define ___swab16(x) \
({ \
	uint16_t __x = (x); \
	((uint16_t)( \
		(((uint16_t)(__x) & (uint16_t)0x00ffU) << 8) | \
		(((uint16_t)(__x) & (uint16_t)0xff00U) >> 8) )); \
})

#define ___swab32(x) \
({ \
	uint32_t __x = (x); \
	((uint32_t)( \
		(((uint32_t)(__x) & (uint32_t)0x000000ffUL) << 24) | \
		(((uint32_t)(__x) & (uint32_t)0x0000ff00UL) <<  8) | \
		(((uint32_t)(__x) & (uint32_t)0x00ff0000UL) >>  8) | \
		(((uint32_t)(__x) & (uint32_t)0xff000000UL) >> 24) )); \
})

#define ___swab64(x) \
({ \
	uint64_t __x = (x); \
	((uint64_t)( \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000000000ffULL) << 56) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000000000ff00ULL) << 40) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000000000ff0000ULL) << 24) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000ff000000ULL) <<  8) | \
	        (uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000ff00000000ULL) >>  8) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000ff0000000000ULL) >> 24) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00ff000000000000ULL) >> 40) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0xff00000000000000ULL) >> 56) )); \
})

/* Swap endian conversion functions to use */
#if __BYTE_ORDER == __BIG_ENDIAN
#define cpu_to_le64(x) ___swab64((x))
#define le64_to_cpu(x) ___swab64((x))
#define cpu_to_le32(x) ___swab32((x))
#define le32_to_cpu(x) ___swab32((x))
#define cpu_to_le16(x) ___swab16((x))
#define le16_to_cpu(x) ___swab16((x))
#else
#define cpu_to_le64(x) (x)
#define le64_to_cpu(x) (x)
#define cpu_to_le32(x) (x)
#define le32_to_cpu(x) (x)
#define cpu_to_le16(x) (x)
#define le16_to_cpu(x) (x)
#endif

#endif /* cpu_to_le32 */

#endif

